"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const zen_push_1 = __importDefault(require("@zkochan/zen-push"));
const reportError_1 = __importDefault(require("../reportError"));
const formatWarn_1 = __importDefault(require("./utils/formatWarn"));
const zooming_1 = require("./utils/zooming");
const os = require("os");
const most = require("most");
// eslint-disable:object-literal-sort-keys
const LOG_LEVEL_NUMBER = {
    error: 0,
    warn: 1,
    info: 2,
    debug: 3,
};
// eslint-enable:object-literal-sort-keys
const MAX_SHOWN_WARNINGS = 5;
exports.default = (log$, opts) => {
    var _a, _b;
    const maxLogLevel = (_b = LOG_LEVEL_NUMBER[(_a = opts.logLevel) !== null && _a !== void 0 ? _a : 'info']) !== null && _b !== void 0 ? _b : LOG_LEVEL_NUMBER['info'];
    const reportWarning = makeWarningReporter(opts);
    return most.merge(log$.registry, log$.other)
        .filter((obj) => LOG_LEVEL_NUMBER[obj.level] <= maxLogLevel &&
        (obj.level !== 'info' || !obj['prefix'] || obj['prefix'] === opts.cwd))
        .map((obj) => {
        var _a;
        switch (obj.level) {
            case 'warn': {
                return reportWarning(obj);
            }
            case 'error':
                if (((_a = obj['message']) === null || _a === void 0 ? void 0 : _a['prefix']) && obj['message']['prefix'] !== opts.cwd) {
                    return most.of({
                        msg: `${obj['message']['prefix']}:` + os.EOL + reportError_1.default(obj, opts.config),
                    });
                }
                return most.of({ msg: reportError_1.default(obj, opts.config) });
            default:
                return most.of({ msg: obj['message'] });
        }
    });
};
// Sometimes, when installing new dependencies that rely on many peer dependencies,
// or when running installation on a huge monorepo, there will be hundreds or thousands of warnings.
// Printing many messages to the terminal is expensive and reduces speed,
// so pnpm will only print a few warnings and report the total number of the unprinted warnings.
function makeWarningReporter(opts) {
    let warningsCounter = 0;
    let collapsedWarnings;
    return (obj) => {
        warningsCounter++;
        if (warningsCounter <= MAX_SHOWN_WARNINGS) {
            return most.of({ msg: zooming_1.autozoom(opts.cwd, obj.prefix, formatWarn_1.default(obj.message), opts) });
        }
        const warningMsg = formatWarn_1.default(`${warningsCounter - MAX_SHOWN_WARNINGS} other warnings`);
        if (!collapsedWarnings) {
            collapsedWarnings = new zen_push_1.default();
            // For some reason, without using setTimeout, the warning summary is printed above the rest of the warnings
            // Even though the summary event happens last. Probably a bug in "most".
            setTimeout(() => collapsedWarnings.next({ msg: warningMsg }), 0);
            return most.from(collapsedWarnings.observable);
        }
        setTimeout(() => collapsedWarnings.next({ msg: warningMsg }), 0);
        return most.never();
    };
}
//# sourceMappingURL=reportMisc.js.map